import pygame
import os
import sys
import image_scrolling
import math
import subprocess
from tkinter import messagebox, filedialog, simpledialog
import tkinter as tk
import json
from rgb import RGB
from data import (
    ae_images_position_x, 
    ae_images_position_y, 
    ae_images_path, 
    ae_images_direction, 
    ae_images_size_x, 
    ae_images_size_y, 
    ae_next_id, 
    ae_images_id, 
    ae_images_type, 
    ae_every_frame_position_x, 
    ae_every_frame_position_y, 
    ae_every_frame_path, 
    ae_every_frame_direction, 
    ae_every_frame_size_x, 
    ae_every_frame_size_y, 
    ae_every_frame_type
)

class AnimationEngine:
    """
    Just use:
        var = AnimationEngine()
    and that's it
    
    
    直接使用：
        var = AnimationEngine()
    就行了
    """
    def __init__(self):
        ...