from setuptools import setup

setup(
    name="2D-Animation-lib",
    version="1.4.4",
    packages=["Animation"], 
    package_data={
        "":["LICENSE"], 
        "Animation":[
            "py.typed", 
            "*.py", 
            "*.pyi", 
            "examples/*.png", 
            "method/*.py", "method/*.pyi", 
            "font/*.TTC"
        ]
    }, 
    include_package_data=True, 
    install_requires=["pygame>=2.0.0"],
    author="WatermelonCode",
    description="A 2D simple animation module for pygame.",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    license="MIT", 
    entry_points={
        "console_scripts":[
            "run-Animation-example=Animation.__main__:main", 
            "run-Animation-example-camera=Animation.__main__:camera_demo"
        ]
    }
)